C$Procedure CVT02 ( Convert type 2 OEM )
 
      SUBROUTINE CVT02 ( SETUP, INPUT, OUTPUT, APPEND, CMTFIL )
 
C$ Abstract
C
C     Convert format version 2 OEM file to SPK format.
C
C$ Disclaimer
C
C     THIS SOFTWARE AND ANY RELATED MATERIALS WERE CREATED BY THE
C     CALIFORNIA INSTITUTE OF TECHNOLOGY (CALTECH) UNDER A U.S.
C     GOVERNMENT CONTRACT WITH THE NATIONAL AERONAUTICS AND SPACE
C     ADMINISTRATION (NASA). THE SOFTWARE IS TECHNOLOGY AND SOFTWARE
C     PUBLICLY AVAILABLE UNDER U.S. EXPORT LAWS AND IS PROVIDED "AS-IS"
C     TO THE RECIPIENT WITHOUT WARRANTY OF ANY KIND, INCLUDING ANY
C     WARRANTIES OF PERFORMANCE OR MERCHANTABILITY OR FITNESS FOR A
C     PARTICULAR USE OR PURPOSE (AS SET FORTH IN UNITED STATES UCC
C     SECTIONS 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE
C     SOFTWARE AND RELATED MATERIALS, HOWEVER USED.
C
C     IN NO EVENT SHALL CALTECH, ITS JET PROPULSION LABORATORY, OR NASA
C     BE LIABLE FOR ANY DAMAGES AND/OR COSTS, INCLUDING, BUT NOT
C     LIMITED TO, INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY KIND,
C     INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
C     REGARDLESS OF WHETHER CALTECH, JPL, OR NASA BE ADVISED, HAVE
C     REASON TO KNOW, OR, IN FACT, SHALL KNOW OF THE POSSIBILITY.
C
C     RECIPIENT BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF
C     THE SOFTWARE AND ANY RELATED MATERIALS, AND AGREES TO INDEMNIFY
C     CALTECH AND NASA FOR ALL THIRD-PARTY CLAIMS RESULTING FROM THE
C     ACTIONS OF RECIPIENT IN THE USE OF THE SOFTWARE.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     OEM2SPKP
C
C$ Declarations
 
      IMPLICIT NONE
 
      CHARACTER*(*)         SETUP
      CHARACTER*(*)         INPUT
      CHARACTER*(*)         OUTPUT
      LOGICAL               APPEND
      CHARACTER*(*)         CMTFIL
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     SETUP      I   Name of setup file.
C     INPUT      I   Name of input OEM file.
C     OUTPUT     I   Name of output SPK file.
C     APPEND     I   Append flag.
C     CMTFIL     I   Name of comment file.
C
C$ Detailed_Input
C
C     SETUP          is the name of an OEM2SPK setup file.
C
C     INPUT          is the name of an OEM file to be converted
C                    to SPK format.  This file conforms to the 
C                    format specification given by the 
C                    draft CCSDS Orbit Data Messages Red Book
C                    Acton 502-H.
C
C     OUTPUT         is the name of an SPK file to be written.
C                    OUTPUT may be a new or existing file.
C                    OUTPUT may contain type 13 or type 9 segments.
C
C     APPEND         is a logical flag indicating whether to 
C                    append data to an existing output file.
C                    If APPEND is .TRUE., the file designated 
C                    by OUTPUT must exist.  If APPEND is .FALSE.,
C                    the file designated by OUTPUT must not exist.
C
C     CMTFIL         is the name of a comment file whose contents
C                    are to be added to the comment area of
C                    the SPK file created by this program.  The
C                    comment file contents precede the default
C                    comments added by OEM2SPK.
C
C                    If no comment file is specified, CMTFIL is
C                    returned blank.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) Errors detected while reading the OEM file will be 
C        diagnosed by routines called by this routine.
C
C     2) Errors detected while parsing the OEM file will be
C        diagnosed by routines called by this routine.
C
C     3) Errors detected while opening the output SPK file will be
C        diagnosed by routines called by this routine.
C
C     4) Errors detected while writing the output SPK file will be
C        diagnosed by routines called by this routine.
C
C$ Files
C
C     See the descriptions of input arguments INPUT and OUTPUT above.
C
C$ Particulars
C
C     This routine operates by side effects.  The principal actions
C     taken by this routine are:
C
C        1) Open the output file as a new file, or for appending, if
C           the file already exists and appending was requested.
C
C        2) Add comments to the output file.  These include:
C
C             - The contents of a comment file, if any, specified in
C               the setup file.
C
C             - The run time and date, the names of the setup, input,
C               and output files, and an indication of whether the
C               output file was new or appended to.
C
C             - The contents of the setup file.
C
C        3) Read and parse each metadata block and following data
C           block.  Each data block is mapped to one or more SPK
C           segments.
C
C        4) Read and parse one or more covariance blocks. These begin 
C           with the 'COVARIANCE_START' keyword and ends with
C           'COVARIANCE_STOP' keyword. Covariances exist only in version
C           2.0 OEM files.
C
C        5) Close the output file.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     [1]  CCSDS Orbit Data Messages Blue Book, version CCSDS 502.0-B-1,
C          September, 2004.
C
C     [2]  CCSDS Orbit Data Messages Blue Book, version CCSDS 502.0-B-2,
C          November, 2009.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     M.C. Kim        (JPL)
C     B.V. Semenov    (JPL)
C
C$ Version
C
C-    OEM2SPK Version 2.2.0, 30-NOV-2016 (BVS)
C
C        Changed logic to accumulate and write covariance matrices
C        to the comment area in larger chunks.
C
C        Made writing of covariance matrices to the comments dependent
C        on the flag set in the setup file.
C
C-    OEM2SPK Version 2.1.0, 08-MAY-2014 (BVS)
C
C        Bug fix: added initialization of comment buffer cell.
C
C        Bug fix: added check for comment buffer overflow.
C
C        Bug fix: added FAILED() checks after calls to all routines
C        that are not error free.
C
C-    OEM2SPK Version 2.0.0, 30-AUG-2013 (MCK)
C
C        Modified to place covariances in SPK comments.
C
C-    OEM2SPK Version 1.0.0, 26-FEB-2005 (NJB)
C
C-&


C
C     SPICELIB functions
C
      LOGICAL               EXISTS
      LOGICAL               FAILED
      LOGICAL               RETURN
      INTEGER               RTRIM
      INTEGER               WDCNT

C
C     Local parameters
C
      INTEGER               LNSIZE
      PARAMETER           ( LNSIZE = 80 )

      INTEGER               LLSIZE
      PARAMETER           ( LLSIZE = 255 )

      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )

      INTEGER               MXCMNT
      PARAMETER           ( MXCMNT = 100000 )

C
C     Local variables
C
      CHARACTER*(LLSIZE)    CARD
      CHARACTER*(LLSIZE)    CMNBUF ( LBCELL: MXCMNT )
      CHARACTER*(LNSIZE)    DEFMTH
      CHARACTER*(LNSIZE)    EPOCH
      CHARACTER*(LNSIZE)    FRAME
      CHARACTER*(LNSIZE)    LINE
      CHARACTER*(LNSIZE)    METHOD
      CHARACTER*(LNSIZE)    TIMSYS
      CHARACTER*(LNSIZE)    TYPE

      DOUBLE PRECISION      QUOLIM
      DOUBLE PRECISION      SPCLIM
      DOUBLE PRECISION      SPKBEG
      DOUBLE PRECISION      SPKEND
      DOUBLE PRECISION      TBEG
      DOUBLE PRECISION      TEND

      INTEGER               BODY
      INTEGER               CENTER
      INTEGER               DEFDEG
      INTEGER               DEGREE
      INTEGER               HANDLE
      INTEGER               L
      INTEGER               LNUM

      LOGICAL               DONE
      LOGICAL               LOCAPP
      LOGICAL               WRTCOV

C
C     Standard SPICE error handing.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF

      CALL CHKIN ( 'CVT02' )

C
C     Initialize comment buffer cell.
C
      CALL SSIZEC ( MXCMNT, CMNBUF )

C
C     Get the required items from the setup file:
C
C        - Minimum allowed spacing of states within a segment
C        - Maximum quotient of consecutive state separations
C          within a segment
C        - SPK time bounds
C     
C     Also get default values, if available:
C
C        - Interpolation method
C        - Polynomial degree
C
C     Also get the ``copy covariances to comments'' flag.
C
      CALL GETINT ( DEFMTH, DEFDEG, SPCLIM, QUOLIM, SPKBEG, SPKEND,
     .              WRTCOV )

C
C     Capture a local copy of the append flag, since we 
C     update the flag if appending to a non-existent file
C     is requested.
C
      LOCAPP = APPEND
C
C
C     Open the output file for appending or as a new file.
C
      IF ( LOCAPP .AND. EXISTS(OUTPUT) ) THEN

         CALL SPKOPA ( OUTPUT, HANDLE )

      ELSE

         LOCAPP = .FALSE.
         CALL SPKOPN ( OUTPUT, OUTPUT, 0, HANDLE )

      END IF

      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'CVT02' )
         RETURN
      END IF

C
C     Add comments to the output file.
C
      CALL ADDCOM ( HANDLE, SETUP, INPUT, OUTPUT, CMTFIL, LOCAPP )

      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'CVT02' )
         RETURN
      END IF

C
C     Initialize count of comment buffer lines. Start additional
C     comments with a blank line, for backward compatibility 
C     with the version 2.1.0 that used SPCACB.
C     
      L = 1
      CMNBUF ( L ) = ' '

C
C     Read the input file and process each line.
C
      CALL RDLIN ( INPUT, LINE, LNUM, TYPE, DONE ) 

      DO WHILE ( .NOT. DONE )

         IF ( TYPE .EQ. 'COVARIANCE_START_LINE' ) THEN

C
C           Read covariances and, if requested, place them in SPK
C           comments.
C
            CALL PUSHLN

            IF ( FAILED() ) THEN
               CALL CHKOUT ( 'CVT02' )
               RETURN
            END IF

            DO WHILE( TYPE .NE. 'COVARIANCE_STOP_LINE' )

               CALL RDLIN ( INPUT, CARD, LNUM, TYPE, DONE ) 

               IF ( FAILED() ) THEN
                  CALL CHKOUT ( 'CVT02' )
                  RETURN
               END IF

               IF ( DONE ) THEN

                  CALL SETMSG ( 'Format error in OEM file <#>: '//
     .                          'COVARIANCE_STOP line not found.')
                  CALL ERRCH  ( '#', INPUT                       )
                  CALL SIGERR ( 'SPICE(INVALIDFORMAT)'           )
                  CALL CHKOUT ( 'CVT02'                          )
                  RETURN

               END IF

C
C              If there is one slot or no slots available in the
C              buffer, write the buffer contents to the comment area.
C
               IF (  L .GE. MXCMNT - 1 ) THEN

                  CALL SCARDC ( L, CMNBUF )

                  IF ( FAILED() ) THEN
                     CALL CHKOUT ( 'CVT02' )
                     RETURN
                  END IF

                  IF ( WRTCOV ) THEN

                     CALL SPCACC ( HANDLE, CMNBUF )

                     IF ( FAILED() ) THEN
                        CALL CHKOUT ( 'CVT02' )
                        RETURN
                     END IF

                  END IF
                   
                  L = 0

               END IF

C
C              Store line in the buffer.
C
               L = L + 1

               IF ( L .LT. MXCMNT ) THEN

                  CMNBUF ( L ) = CARD ( : RTRIM(CARD) )

C
C                 Insert a blank line after 'COVARIANCE_START',
C                 'COVARIANCE_STOP', and each covariance matrices.
C                  
                  IF ( TYPE .EQ. 'COVARIANCE_START_LINE' .OR. 
     .                 TYPE .EQ. 'COVARIANCE_STOP_LINE'  .OR. 
     .                 WDCNT( CARD ) .EQ. 6                   ) THEN

                     L = L + 1
                     CMNBUF ( L ) = ' '

                  END IF

               ELSE

C
C                 This is a safety check. We should never get here.
C
                  CALL SETMSG ( 'Comment buffer overflow. This '   //
     .                          'should never happen and '         //
     .                          'indicates a bug in the program. ' //
     .                          'Please contact NAIF '    )
                  CALL ERRINT ( '#', MXCMNT                        )
                  CALL ERRCH  ( '#', CARD                          )
                  CALL SIGERR ( 'SPICE(CMTBUFOVERFLOW)'            )
                  CALL CHKOUT ( 'CVT02'                            )
                  RETURN

               END IF

            END DO

         ELSE IF ( TYPE .EQ. 'METADATA_START_LINE' ) THEN

C
C           Parse the following metadata block.
C
            CALL PRSMET ( INPUT,  BODY,  CENTER,  FRAME,  EPOCH,
     .                    TBEG,   TEND,  TIMSYS,  METHOD, DEGREE  )

            IF ( FAILED() ) THEN
               CALL CHKOUT ( 'CVT02' )
               RETURN
            END IF

C
C           If an interpolation method and degree were not specified
C           in the metadata block from the OEM file, defaults from
C           the setup file must be used.
C
            IF ( METHOD .EQ. ' ' ) THEN

               METHOD = DEFMTH
               DEGREE = DEFDEG

            END IF

C
C           The metadata stop line will have been read at this point.
C           The data lines following the metadata block may be
C           processed.  We'll create one or more segments from these
C           data lines. 
C
C           We call MAKSEG even when the metadata block indicates
C           the coverage of the block is outside the interval
C           [SPKBEG, SPKEND].  This is because we need to execute
C           the input and parsing functions of MAKSEG.
C
            CALL MAKSEG ( INPUT,  HANDLE, METHOD, DEGREE,  
     .                    SPCLIM, QUOLIM, BODY,   CENTER, 
     .                    FRAME,  TBEG,   TEND,   TIMSYS,
     .                    SPKBEG, SPKEND                  )

            IF ( FAILED() ) THEN
               CALL CHKOUT ( 'CVT02' )
               RETURN
            END IF

C
C           MAKSEG stops processing when it encounters a new 
C           metadata start line or the end of the input file.
C           In the former case, MAKSEG pushes the line back on
C           the input stream, so the line can be picked up in
C           this routine.
C
         END IF

C
C        At this point RDLIN should return a metadata start line
C        or else indicate and EOF condition.
C
         CALL RDLIN ( INPUT, LINE, LNUM, TYPE, DONE )

         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'CVT02' )
            RETURN
         END IF

      END DO

C
C     Flush comments buffer if we need to.
C     
      IF (    L .GT. 1                                .OR. 
     .     (  L .EQ. 1 .AND. CMNBUF ( L ) .NE. ' ' )       ) THEN

         CALL SCARDC ( L, CMNBUF )

         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'CVT02' )
            RETURN
         END IF

         IF ( WRTCOV ) THEN

            CALL SPCACC ( HANDLE, CMNBUF )

            IF ( FAILED() ) THEN
               CALL CHKOUT ( 'CVT02' )
               RETURN
            END IF

         END IF

      END IF

C
C     The SPK file is complete.
C
      CALL SPKCLS ( HANDLE )

      CALL CHKOUT ( 'CVT02' )
      RETURN
      END
